--- ========================================================= ---
---  Setup Locals
--- ========================================================= ---
local DaHud = LibStub("AceAddon-3.0"):GetAddon("DaHud")
local mod = DaHud:GetModule("Buffs")

local _G = getfenv(0)

local newList, delList = DaHud.newList, DaHud.delList

--- ========================================================= ---
---  Local Functions
--- ========================================================= ---
local auraSort__isFriend
local auraSort__isBuff
local function auraSort(alpha, bravo)
--
-- return true if alpha is before bravo (IE: alpha > bravo)
--

if not mod then
	DEFAULT_CHAT_FRAME:AddMessage("Buffs module not found.")
end

	if not alpha then
		return false
	elseif not bravo then
		return true
	end

	-- show your own buffs/debuffs first
	local alpha_isMine, bravo_isMine = alpha[8], bravo[8]
	if alpha_isMine ~= bravo_isMine then
		if alpha_isMine=="player" then
			return truedd
		else
			return false
		end
	end

-- alpha[8],bravo[8] is isMine attribute
-- apparently sort item buffs before others.  Right now this is just putting self buffs before others

	if auraSort__isBuff then
		-- item buffs first
		local alpha_itemSlot, bravo_itemSlot = alpha[8], bravo[8]
		if alpha_itemSlot and not bravo_itemSlot then
			return true
		elseif not alpha_itemSlot and bravo_itemSlot then
			return false
		elseif alpha_itemSlot and bravo_itemSlot then
			return alpha_itemSlot < bravo_itemSlot
		end
	else
	
-- apparently put friendly buffs before others.  If it isn't already determined to be a self buff.

		if auraSort__isFriend then
			-- sort by dispel type
			local alpha_dispelType, bravo_dispelType = alpha[5], bravo[5]
			if alpha_dispelType ~= bravo_dispelType then
				if not alpha_dispelType then
					return false
				elseif not bravo_dispelType then
					return true
				end
				local canDispel_alpha_dispelType = mod.Spells.canDispel[alpha_dispelType]
				if not canDispel_alpha_dispelType ~= not mod.Spells.canDispel[bravo_dispelType] then
					-- show debuffs you can dispel first
					if canDispel_alpha_dispelType then
						return true
					else
						return false
					end
				end
				return alpha_dispelType < bravo_dispelType
			end
		end
	end

-- sort by name

	local alpha_name, bravo_name = alpha[2], bravo[2]
	if alpha_name ~= bravo_name then
		if not alpha_name then
			return false
		elseif not bravo_name then
			return true
		end
		if auraSort__isBuff then
			if auraSort__isFriend then
				-- show buffs you can cast first
				local alpha_isMyBuff = alpha[8]
				local bravo_isMyBuff = bravo[8]

				if not alpha_isMyBuff ~= not bravo_isMyBuff then
					if alpha_isMyBuff then
						return true
					else
						return false
					end
				end
			end
		else
			if auraSort__isFriend then
				-- show debuffs you can cast first
				local alpha_isMyBuff = alpha[8]
				local bravo_isMyBuff = bravo[8]
				if not alpha_isMyDebuff ~= not bravo_isMyDebuff then
					if alpha_isMyDebuff then
						return true
					else
						return false
					end
				end
			else
				local alpha_isMyBuff = alpha[8]
				local bravo_isMyBuff = bravo[8]
				if not alpha_isMyDebuff ~= not bravo_isMyDebuff then
					if alpha_isMyDebuff then
						return true
					else
						return false
					end
				end
			end
		end
		return alpha_name < bravo_name
	end

	-- keep ID order
	local alpha_id, bravo_id = alpha[1], bravo[1]
	if not alpha_id then
		return false
	elseif not bravo_id then
		return true
	end
	return alpha_id < bravo_id
end

mod.auraSort = auraSort
mod.auraSort__isFriend = auraSort__isFriend
mod.auraSort__isBuff = auraSort__isBuff

local canDispel
local _, playerClass = _G.UnitClass('player')
local _, playerRace = _G.UnitRace('player')

-- ============================
-- Streamlining the update requirements for this module we no longer need selfbuffs/debuffs, nor 
-- friendbuffs/enemybuffs/frienddebuffs/enemydebuffs.  This is handled through the isMine return from the
-- new UnitBuffs() functionality.
-- We do need to keep the canDispel arrays per class, but those are shorter and not likely to be changed.
-- ============================
if playerClass == 'PRIEST' then
	canDispel = {
		["Magic"] = true,
		["Disease"] = true,
	}
	
elseif playerClass == 'SHAMAN' then
	canDispel = {
		["Poison"] = true,
		["Disease"] = true,
		["Curse"] = true,
	}
elseif playerClass == 'PALADIN' then
	canDispel = {
		["Magic"] = true,
		["Poison"] = true,
		["Disease"] = true,
	}
elseif playerClass == 'MAGE' then
	canDispel = {
		["Curse"] = true,
	}
elseif playerClass == 'DRUID' then
	canDispel = {
		["Curse"] = true,
		["Poison"] = true,
	}
elseif playerClass == 'WARRIOR' then
elseif playerClass == 'WARLOCK' then
	canDispel = {
		["Magic"] = true,
	}
elseif playerClass == 'HUNTER' then
	canDispel = {
		["Magic"] = true,
	}
elseif playerClass == 'ROGUE' then
end

if not canDispel then
	canDispel = {}
end

local function turn(t)
	local tmp = newList()
	for k in _G.pairs(t) do
		local spell = (_G.GetSpellInfo(k))
		if not spell then
			error(("Unknown spell ID: %d"):format(k))
		end
		tmp[spell] = true
		t[k] = nil
	end
	for k in _G.pairs(tmp) do
		t[k] = true
		tmp[k] = nil
	end
	tmp = delList(tmp)
end

mod.Spells = {
	canDispel = canDispel,
}
