--- ========================================================= ---
---  Setup Locals
--- ========================================================= ---
local L = LibStub("AceLocale-3.0"):GetLocale("DaHud", true)
local DaHud = LibStub("AceAddon-3.0"):GetAddon("DaHud")
local mod = DaHud:GetModule("Bars", true)

if not(mod) then return end

local localeTables = {}
function DaHud:L(name)
	if not localeTables[name] then
		localeTables[name] = setmetatable({}, {__index = function(self, key)
			self[key] = key
			return key
		end})
	end
	return localeTables[name]
end

local L = DaHud:L("DaHud")

--- ========================================================= ---
---  API
--- ========================================================= ---
DaHud.Bars.Plugins.reputation = L["Reputation"]

DaHud.Bars.Type.player.reputation = L["Reputation"]

--- ========================================================= ---
---  Create Frame Settings
--- ========================================================= ---
function mod:CreateFrameSettings_reputation(id)
	if not(DaHud.db.profile.colors.other) then
		DaHud.db.profile.colors.other = {}
	end
	
	if not(DaHud.db.profile.colors.other.reputation) then
		DaHud.db.profile.colors.other.reputation = {0, 0.75, 0.75}
	end
end

--- ========================================================= ---
---  Create Frame Options
--- ========================================================= ---
function mod:CreateFrameOptions_reputation()
	if not(DaHud.options.args.general.args.colors.args.other) then
		DaHud.options.args.general.args.colors.args.other ={
			name = L["Other"],
			type= "group",
			order = 6,
			args = {
			},
		}
	end
	
	local path = DaHud.options.args.general.args.colors.args
	DaHud:AddColorOption(path, L["Reputation"], 1, "other", "reputation")
end

--- ========================================================= ---
---  Register Frame Events
--- ========================================================= ---
function mod:RegisterFrameEvents_reputation(id)
	if not(id) then return end
	
	local f = DaHud.Frames.Bars[id]
	
	local settings = DaHud.db.profile.bars[id]
	
	if (settings.type ~= "reputation") then return end
	
	if settings.unit == "player" then
		f:RegisterEvent("PLAYER_ENTERING_WORLD")
	end
	
	f:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
	f:RegisterEvent("UPDATE_FACTION")
end

--- ========================================================= ---
---  Update Reputation
--- ========================================================= ---
function mod:Update_reputation(f)
	if not(f) then return end

	local id = string.gsub(f:GetName(), "DaHud_", "")
	local settings = DaHud.db.profile.bars[id]
	
	local r, g, b = unpack(DaHud.db.profile.colors.other.reputation)
	
	local repname, repreaction, repmin, repmax, repvalue = GetWatchedFactionInfo()
	local current, max = repvalue - repmin, (repmax - repmin)

	if not(max) or max == 0 then
		current = 0
		max = 1
	else
		current = current/max
	end
	
	f.Background:SetVertexColor(r, g, b, 0.4)
	f.Bar:SetVertexColor(r, g, b, 1)
	f.StatusBar:SetStatusBarColor(r, g, b, 1)
	
	if not(settings.animate) then
		f.currentValue = current
		f.targetValue = current
	else
		f.targetValue = current

		if f.currentValue ~= f.targetValue then
			f.animationEndTime = GetTime() + 0.5
			f.animating = true
		end
	end
end